clear all; clc; pkg load statistics;
%Load set of coordinates of nodes from test instance file:
data=textread('SJC1.dat','%s','delimiter','\n'); 
coordinates=[]; demands=[];
for i=2:length(data)
    [coordx coordy capacity dem]=strread(data{i},'%f %f %d %d');
    coordinates=[coordinates;coordx coordy]; demands=[demands;dem];
end
P = 10; % Number of centroids - facilities
N = length(coordinates(:,1)); % Number of clients - nodes
% Generate random solution
[centroids assignments total_distance] = random_assignment(N,P,capacity, demands, coordinates);
REF = total_distance; % Initial reference value for assessment of convergence
fprintf('Iteration 0 with REF = %1.5f \n',REF);
ITERATIONS=50; convergence=[];
% Local Search Metaheuristic for Improvement
for i=1:ITERATIONS
    [centroids assignments total_distance] = update_centroids_assignment(N,P,capacity, demands, coordinates, assignments);
    if total_distance < REF
        best_assignments=assignments; best_centroids=centroids; REF=total_distance;
        fprintf('Iteration %d with REF = %1.5f \n', i, REF);
    else
        if total_distance == Inf
            [centroids assignments total_distance] = random_assignment(N,P,capacity, demands, coordinates);
        end
    end
    convergence=[convergence;REF];
end
% Verification Routine
dems=[]; dists=[];
for i=1:P
    dem_ac=0; coords=[];
    for j=1:N
        if best_assignments(j)==i
            coords=[coords;coordinates(j,:)]; dem_ac=dem_ac+demands(j);
        end
    end
    dems=[dems;dem_ac]; dist=0; cx=best_centroids(i,:);
    for j=1:length(coords(:,1))
        dist=dist+ sqrt ( (cx(1)-coords(j,1))^2 + (cx(2)-coords(j,2))^2);
    end
    dists=[dists; dist];
end
dists=sum(dists);
if length(find(dems>capacity))>0
    fprintf('Error \n');
else
    if dists == REF fprintf('Full Success \n'); else fprintf('Partial Success \n'); end
end
figure; hold on;
for i=1:P
    coords=[];
    % Identify nodes assigned to facility i
    for j=1:N
        if best_assignments(j)==i
            coordsx=[best_centroids(i,1) coordinates(j,1)]; coordsy=[best_centroids(i,2) coordinates(j,2)];
            plot(coordsx,coordsy,'-b');
        end
    end
end
title('Assignments of Clients to Facilities','FontSize',12);
xlabel('X-Coordinates'); ylabel('Y-Coordinates');
figure; hold on;
title('Convergence of Local Search Metaheuristic','FontSize',12);
plot(1:1:ITERATIONS, convergence, 'ob');
xlabel('Iterations'); ylabel('Total Distance of Solution');