function [centroids assignments total_distance] = random_assignment(N,P,capacity, demands, coordinates)
% P = number of facilities, N = number of nodes
% Generate random solution
x_min=min(coordinates(:,1)); x_max=max(coordinates(:,1));
y_min=min(coordinates(:,2)); y_max=max(coordinates(:,2));
% Random centroids within the range of the nodes' coordinates
centroids=[];
for i=1:P
    centroids=[centroids; unifrnd(x_min,x_max) unifrnd(y_min,y_max)];
end
% Initialization of arrays
distance_matrix = zeros(P,N); % Distance matrix PxN
assignments = zeros(1,N); % Assignment vector for each node within N
capacities = zeros(1,P); % Cumulative capacity vector for each facility within P
distances = zeros(1,P); % Cumulative distance vector for each facility within P
% Compute distance matrix PxN
for i=1:P
    pA=centroids(i,:);
    for j=1:N
        pB=coordinates(j,:); distance_matrix(i,j) = sqrt((pA(1)-pB(1))^2+(pA(2)-pB(2))^2);
    end
end
clients=1:1:N; % List of all nodes to be assigned to each facility within P (unassigned nodes)
% Perform assignment while there are unassigned nodes
while length(clients)>0
    a=min(min(distance_matrix));
    [row,col]=find(distance_matrix==a); % location of the minimum value within the distance matrix
    if length(row)>1
        % random selection if more than one node has the same minimum distance to a facility
        c=ceil(unifrnd(0,length(row)));
    else
        c=1;
    end
    % Identification of the node with the minimum distance to a facility
    facility = row(c); node = col(c);
    % Perform assignment if the node is within the list of unassigned nodes
    % and there is available capacity within the facility
    if length(find(clients==node))>0 && capacities(facility)+demands(node)<=capacity
        assignments(node)=facility; % Perform assignment of the node to the facility
        capacities(facility)=capacities(facility)+demands(node); % Update capacity of the facility
        d=find(clients==node); clients(d)=[]; % Locate and remove assigned node from the list of unassigned nodes
        distance_matrix(facility,node)=Inf; % Remove the minimum value of the assigned node from the distance matrix
    else
        % If assignment cannot be performed, then remove the minimum value
        % for future assignment to a different facility
        distance_matrix(facility,node)=Inf; % Remove the current minimum value for future assignment
    end
end
% Compute total distance from current centroids to assigned nodes
for i=1:P
    dist=0; coords=[];
    % Identify nodes assigned to facility i
    for j=1:N
        if assignments(j)==i coords=[coords;coordinates(j,:)]; end
    end
    if length(coords)>0
        cx=centroids(i,:);
        % Compute cumulative distance from centroid to all its assigned nodes
        for j=1:length(coords(:,1))
            cy=coords(j,:); dist=dist+sqrt((cx(1)-cy(1))^2+(cx(2)-cy(2))^2);
        end
        distances(i)=dist;
    else
        % If no nodes are assigned to the facility, remove facility by making its distance equal to Inf.
        % This also marks this solution as not valid but suitable for improvement.
        distances(i)=Inf;
    end
end
total_distance=sum(distances);