function [centroids assignments total_distance] = update_centroids_assignment(N,P,capacity, demands, coordinates, assignments)
% P = number of facilities, N = number of nodes / clients
% Initialization of arrays
distance_matrix = zeros(P,N); % Distance matrix PxN
capacities = zeros(1,P); % Cumulative capacity vector for each facility within P
distances = zeros(1,P); % Cumulative distance vector for each facility within P
%Backup of initial assignments
original_assignments = assignments;
%Compute centroids from current assignments
centroids=[];
for i=1:P
    coords=[];
    for j=1:N
        if assignments(j)==i coords=[coords;coordinates(j,:)]; end
    end
    if length(coords)>0
        if length(coords(:,1))==1
            cx=coords; % If only one node is assigned, then this is the centroid's location
        else
            % If more than one node is assigned, their average coordinates are the centroid's location
            if length(coords(:,1))>1 cx=mean(coords); end
        end
    else
        % If no nodes are assigned to the facility, compute the centroid's
        % location as the average of all coordinates
        cx=mean(coordinates);
    end
    centroids=[centroids;cx]; % Update centroids
end
% Compute distance matrix with updated centroids
for i=1:P
    pA=centroids(i,:);
    for j=1:N pB=coordinates(j,:); distance_matrix(i,j)= sqrt((pA(1)-pB(1))^2+(pA(2)-pB(2))^2); end
end
clients=1:1:N; % List of all nodes to be assigned to each facility within P (unassigned nodes)
% Initialization for update of assignments with updated centroids
assignments = zeros(1,N); alarm=0;
while length(clients)>0 && alarm == 0
    a=min(min(distance_matrix));
    if min(min(distance_matrix))==Inf
        alarm = 1;
    else
        [row,col]=find(distance_matrix==a); % location of the minimum value within the distance matrix
        % random selection if more than one node has the same minimum distance to a facility
        if length(row)>1 c=ceil(unifrnd(0,length(row))); else c=1; end
        % Identification of the node with the minimum distance to a facility
        facility = row(c); node = col(c);
        % Perform assignment if the node is within the list of unassigned nodes
        % and there is available capacity within the facility
        if length(find(clients==node))>0 && capacities(facility)+demands(node)<=capacity
            assignments(node)=facility; % Perform assignment of the node to the facility
            capacities(facility)=capacities(facility)+demands(node); % Update capacity of the facility
            % Locate and remove assigned node from the list of unassigned nodes
            d=find(clients==node); clients(d)=[];
            distance_matrix(facility,node)=Inf; % Completely remove assigned node from the distance matrix
        else
            % If assignment cannot be performed, then remove the minimum value
            % for future assignment to a different facility
            distance_matrix(facility,node)=Inf; % Remove the current minimum value for future assignment
        end
    end
end
if alarm==0
    distances=zeros(1,P);
    % Compute total distance from current centroids to assigned nodes
    for i=1:P
        dist=0; coords=[];
        % Identify nodes assigned to facility i
        for j=1:N
            if assignments(j)==i coords=[coords;coordinates(j,:)]; end
        end
        if length(coords)>0
            cx=[]; cx=centroids(i,:);
            % Compute cumulative distance from centroid to all its assigned nodes
            for j=1:length(coords(:,1))
                cy=[]; cy=coords(j,:); dist=dist+sqrt((cx(1)-cy(1))^2+(cx(2)-cy(2))^2);
            end
            distances(i)=dist;
        else
            % If no nodes are assigned to the facility, remove facility by making its distance equal to Inf.
            % This also marks this solution as not valid but suitable for improvement.
            distances(i)=Inf;
        end
    end
    total_distance=sum(distances);
else
    total_distance=Inf;
end
if total_distance==Inf
    fprintf('unfeasible solution - initialize with random centroids \n');
    assignments=original_assignments; centroids=[];
    x_min=min(coordinates(:,1)); x_max=max(coordinates(:,1));
    y_min=min(coordinates(:,2)); y_max=max(coordinates(:,2));
    for i=1:P centroids=[centroids; unifrnd(x_min,x_max) unifrnd(y_min,y_max)]; end
    total_distance=Inf;
end