clear all; clc; pkg load statistics; 
coords=[]; demands=[];
 
%============== Read File of Coordinate and Demand Data  =======================
data=textread('X-n219-k73.vrp','%s','delimiter','\n'); 
[s_1 s_2 nodes]=strread(data{4},'%s %s %d');
[s_1 s_2 capacity]=strread(data{6},'%s %s %d');
for i=8:8+nodes-1 % Lines of File with Coordinates Data
  [node tx ty]=strread(data{i},'%d %f %f');  coords=[coords; tx ty];
end
for i=8+nodes+1:length(data)-4 % Lines of File with Demand Data
  [node d]=strread(data{i},'%d %f');  demands=[demands; d];
end
 
% ============= Compute Euclidean Symmetric Distance Matrix ==================== 
Euc_DM=[]; Euc_DM0=[];
for i=1:length(coords(:,1))
  loc_i=coords(i,:);
  for j=1:length(coords(:,1))
    loc_j=coords(j,:); 
    Euc_DM(i,j)=sqrt((loc_i(1)-loc_j(1))^2+(loc_i(2)-loc_j(2))^2);
  end
end 
 
% ============= Compute Distance Statistics for Sequencing =====================
tp_0=sort(reshape(Euc_DM,1,nodes*nodes)); %Re-arrange into vector and sort
tp_1=tp_0(nodes+1:length(tp_0));      %Eliminate distance data of i=j
mn_dst=mean(tp_1); std_dst=std(tp_1); %Mean and standard deviation of Euc_DM
% ==============================================================================
 
absolute=Inf;
for max_iteras=1:500
  % ======= Nearest-Neighbor NN sequencing to obtain total single route ========== 
  % different weights for the standard deviation
  w_std=[0.00001, 0.00005 0.0001 0.0005 0.001 0.005 0.01 0.05]; 
  cont_lim=1; % counter for selection of weights
  members=2:1:nodes; % vector with all remaining customer locations  
  partial_route=[]; all_nodes_inserted=0;
  current_node=1; % The partial route starts at the "current" node "1"
  while all_nodes_inserted==0
    threshold= std_dst*w_std(cont_lim); cont_lim=cont_lim+1;
    if cont_lim>length(w_std) cont_lim=1; end
    distances=[];
    for j=1:length(members) % Compute distances from "current" node to remaining nodes
      distances(j)=Euc_DM(current_node,members(j));
    end
    if isempty(distances)==0 % There are remaining nodes to be inserted if distances is
      % not empty. Candidate remaining nodes are those within a distance threshold
      ind_temp=find(distances<=(min(distances))+threshold); 
      % The "closest" node is randomly selected from the candidate remaining nodes 
      closest_node=members(ind_temp(ceil(unifrnd(0,length(ind_temp)))));
      ind_ref=find(members==closest_node);
      % (1)"closest" node is removed from the remaining nodes,(2)"current" node is inserted
      % into the "partial" route,(3)"current" node is updated with the "closest" node
      members(ind_ref)=[];partial_route=[partial_route current_node];current_node=closest_node;
    else
      all_nodes_inserted=1; partial_route=[partial_route current_node]; %All nodes inserted
    end
  end
  partial_route(1)=[]; %Node "1" is removed from partial route
  [total_route distance_improved_route]=randomimprovement(partial_route, Euc_DM); 
  [CVRP_distance CVRP_routes]=partitioning2(total_route,capacity,demands,Euc_DM);
 
  if CVRP_distance< absolute
    best_absolute=[];
    best_absolute=CVRP_routes; %Updates best absolute CVRP solution
    absolute=CVRP_distance;    %Updates distance value of the best absolute CVRP solution
    fprintf('%d  CVRP distance = %1.4f \n',max_iteras, absolute);
    best_partial_route=[];
    best_partial_route=total_route;
  end
end

%Plot routine to draw the partial route, the labels of each node, and the
%CVRP sub-routes
figure
hold on
label_nodes= 1:1:nodes; label_text = cellstr(num2str(label_nodes'));
for i=1:nodes 
  plot(coords(i,1),coords(i,2),'*b');
  text(coords(i,1),coords(i,2),label_text(i),'FontSize',10); 
end
 
for i=1:length(best_partial_route)-1
  xx=[coords(best_partial_route(i),1) coords(best_partial_route(i+1),1)];
  yy=[coords(best_partial_route(i),2) coords(best_partial_route(i+1),2)];
  plot(xx,yy,'--k','LineWidth',2);
end
 
figure
hold on
 
for k=1:length(best_absolute(:,1))
  sub_route=[]; sub_route=[1 best_absolute(k,:) 1];
  %Eliminate zeros from sub_route
  find_zeros=find(sub_route==0); sub_route(find_zeros)=[];
  for i=1:length(sub_route)-1
    xx=[coords(sub_route(i),1) coords(sub_route(i+1),1)];
    yy=[coords(sub_route(i),2) coords(sub_route(i+1),2)];
    plot(xx,yy,'-b');
  end
end