clear all; clc; pkg load statistics
d = 10; std = 6; %units per day of average demand and standard deviation
z = 3;           %standard deviations for demand variability 
K = 365;         %working days through the planning horizon
D = K*d;         %cumulative demand
%         C      Co    Ch   p
Costs = [250.0 1250.0 25.0 150.0];  % Inventory Associated Costs
LT = 7; T = 21;  %days of lead time and days between reviews
% ============= Determine parameters for (Q,R) model =========================
for i=1:20 %Number of iterations
  if i==1 % Initial Value for Q 
    Q = sqrt((2*D*Costs(2))/Costs(3)); 
  else 
    Q = sqrt((2*D*(Costs(2) + Costs(4)*n))/Costs(3));
  end
  A=1-Q*(Costs(3)/(Costs(4)*D));  Z=norminv(A); % Z for Continuous Review Model
  R = d*LT + Z*std*sqrt(LT);
  L = stdnormal_pdf(Z)-Z*(1-stdnormal_cdf(Z)); % Loss function
  n = L*std*sqrt(LT);
end
Q=ceil(Q); R=ceil(R);% round final values towards positive infinity
% ============== Determine Associated Total Inventory Cost ===================
TOC = (Costs(2)*D)/Q; % Total Order Cost
THC = (Costs(3)*Q)/2; % Total Holding Cost
TSS = Costs(3)*(R - d*LT + std*sqrt(LT)*L); % Total Holding Cost of Safety Stock
TSK = Costs(4)*A*(D/Q); % Total Stock-Out Cost
TC = TOC +THC +TSS +TSK ;
% =============   Test (Q,R) Parameters  =====================================
Inventory = Q+R; count_LT=0; inv_consumption = []; % Initialization
for i=1:K
  inv_consumption = [inv_consumption ; Inventory];
  dt=d+unifrnd(-z,z)*std; % Random daily demand within [-z, +z] std
  if dt<0 
    dt = 0; 
  end
  if Inventory - dt > R  
    Inventory =Inventory - dt; 
  else 
    Inventory =Inventory - dt; 
    count_LT=count_LT+1;
  end
  if count_LT == LT+1
    Inventory=Inventory+Q; count_LT=0;
  end
end
% =================        Visualization Code       ==========================
vect_R=ones(length(inv_consumption),1)*R;
plot(inv_consumption); hold on;  plot(vect_R,'-r'); axis([0 K 0 Q+R]);
xlabel('Working Days'); ylabel('Inventory level');
text_title='Continuous Review Strategy with [-z,+z], Q = QX, R = RX, and TC = TCX';
text_title=strrep(text_title,'z',mat2str(z));
text_title=strrep(text_title,'QX',mat2str(Q));
text_title=strrep(text_title,'RX',mat2str(R));
text_title=strrep(text_title,'TCX',mat2str(TC));
title(text_title);
