clear all
clc
pkg load statistics

%Read Ccordinate File
data=textread('d493.tsp','%s','delimiter','\n');
old_coordinates=[];

for i=7:length(data)-1
  [node tx ty]=strread(data{i},'%d %f %f');
  old_coordinates=[old_coordinates; tx ty];
end

plot(old_coordinates(:,1),old_coordinates(:,2),'.k'); 
xlabel('x-coordinate');  ylabel('y-coordinate'); grid on;

new_coordinates=zeros(length(old_coordinates(:,1)),2);
% Convert x-coordinates into Longitude Coordinates
max_new=-100; min_new=-102;
max_old=max(old_coordinates(:,1));
min_old=min(old_coordinates(:,1));

new_coordinates(:,1)=((max_new-min_new)/(max_old-min_old))*(old_coordinates(:,1)-max_old)+max_new;

% Convert y-coordinates into Latitude Coordinates
max_new=22; min_new=20;
max_old=max(old_coordinates(:,2));
min_old=min(old_coordinates(:,2));

new_coordinates(:,2)=((max_new-min_new)/(max_old-min_old))*(old_coordinates(:,2)-max_old)+max_new;

plot(new_coordinates(:,1),new_coordinates(:,2),'.b');
xlabel('Longitude ()');  ylabel('Latitude ()'); grid on;

demands=[]; 
capacity=100;   % Capacity of the vehicle or warehouse / distribution center
pct=0.20;       % Percentage of capacity to be considered for generation of demand

for i=1:rows(new_coordinates)
    if i==1
        demands=[demands; 0];
    else
        demands=[demands; ceil(unifrnd(0,pct*capacity))];
    end
end

% Estimate symmetric Euclidean distance matrix
Euc_DM=[];
for i=1:rows(new_coordinates)
  location_i=new_coordinates(i,:);
  for j=i:rows(new_coordinates)
    location_j=new_coordinates(j,:);
    Euc_DM(i,j)=sqrt((location_i(1)-location_j(1))^2+(location_i(2)-location_j(2))^2);
  end
end
Euc_DM=Euc_DM+Euc_DM';

% Estimate symmetric Manhattan distance matrix
MNTN_DM=[];
for i=1:rows(new_coordinates)
  location_i=new_coordinates(i,:);
  for j=i:rows(new_coordinates)
    location_j=new_coordinates(j,:);
    MNTN_DM(i,j)=abs(location_i(1)-location_j(1))+abs(location_i(2)-location_j(2));
  end
end
MNTN_DM=MNTN_DM+MNTN_DM';

% Estimate symmetric Earth's Arc length distance matrix
ARC_DM=[];
R=6371; %Radius of the spherical Earth
for i=1:rows(new_coordinates)
  location_i=(pi/180)*new_coordinates(i,:);
  for j=i:rows(new_coordinates)
    location_j=(pi/180)*new_coordinates(j,:);
    term1=sin(location_i(2))*sin(location_j(2));
    term2=cos(location_i(2))*cos(location_j(2))*cos(location_i(1)-location_j(1));
    ARC_DM(i,j)=acos(term1+term2);
  end
end
ARC_DM=R*(ARC_DM+ARC_DM');































