clear all; clc; pkg load statistics
d = 10; std = 6; %units per day of average demand and standard deviation
z = 7;           %standard deviations for demand variability 
K = 365;         %working days through the planning horizon
D = K*d;         %cumulative demand
SL = 0.985;      %service level of 98.5%
% Inventory associated costs
%         C      Co    Ch   p
Costs = [250.0 1250.0 25.0 150.0];  % Inventory Associated Costs
LT = 7; T = 21;  % days of lead time and days between reviews
Z = norminv(SL); % Z-value for Periodic Review Model
% ============= Determine parameters for P model =========================
I = 0; Q=d*(T+LT)+Z*std*sqrt(T+LT)-I; Inventory=Q; count_T=0; count_LT=0;
% ============== Determine Associated Total Inventory Cost ===================
TOC = (Costs(2)*D)/(d*T); % Total Order Cost
THC = (Costs(3)*(d*T))/2; % Total Holding Cost
TSS = Costs(3)*Z*std*sqrt(T+LT); % Total Holding Cost of Safety Stock
TC = TOC +THC +TSS;
% =============   Test P Parameters  =====================================
inv_consumption = [];
for i=1:K
  inv_consumption = [inv_consumption ; Inventory];
  dt=d+unifrnd(-z,z)*std; % Random daily demand within [-z, +z] std
  if dt<0 
    dt = 0; 
  end
  Inventory=Inventory-dt; count_T=count_T+1;
  if count_T == T  
    I= Inventory; count_T=0;  count_LT=LT+1;  
  end
  if count_LT>0
    count_LT=count_LT-1;
    if count_LT == 0
      Inventory = Inventory+(Q-I); 
    end
  end
end
% =================        Visualization Code       ==========================
plot(inv_consumption); hold on; axis([0 K 0 Q]);
xlabel('Working Days'); ylabel('Inventory level');
text_title='Periodic Review Strategy with [-z,+z], Q = QX and TC = TCX';
text_title=strrep(text_title,'z',mat2str(z));
text_title=strrep(text_title,'QX',mat2str(Q));
text_title=strrep(text_title,'TCX',mat2str(TC));
title(text_title);
